@@ -3,6 +3,7 @@
 import concurrent.futures
 import glob
 from importlib import metadata
+import ipaddress
 from itertools import groupby
 import os
 import platform
@@ -71,6 +72,36 @@ settings.validators.register(
     ),
 )
 
+# Define device roles that should get Loopback0 interfaces
+NETBOX_NODE_ROLES = [
+    "compute",
+    "storage",
+    "resource",
+    "control",
+    "manager",
+    "network",
+    "metalbox",
+    "dpu",
+    "loadbalancer",
+    "router",
+    "firewall",
+]
+
+# Define switch roles that should also get Loopback0 interfaces
+NETBOX_SWITCH_ROLES = [
+    "accessleaf",
+    "borderleaf",
+    "computeleaf",
+    "dataleaf",
+    "leaf",
+    "serviceleaf",
+    "spine",
+    "storageleaf",
+    "superspine",
+    "switch",
+    "transferleaf",
+]
+
 
 def validate_netbox_connection():
     """Validate NetBox connection settings."""
@@ -151,10 +182,9 @@ def load_global_vars() -> dict:
     """Load and merge global variables from the VARS directory."""
     global_vars: dict[str, Any] = {}
 
-    if not settings.VARS:
+    vars_dir = getattr(settings, "VARS", None)
+    if not vars_dir:
         return global_vars
-
-    vars_dir = settings.VARS
     if not os.path.exists(vars_dir):
         logger.debug(f"VARS directory {vars_dir} does not exist, skipping global vars")
         return global_vars
@@ -511,8 +541,9 @@ def _run_main(
 
         # Find files in numbered subdirectories (excluding vars directory)
         vars_dirname = None
-        if settings.VARS:
-            vars_dirname = os.path.basename(settings.VARS)
+        vars_dir = getattr(settings, "VARS", None)
+        if vars_dir:
+            vars_dirname = os.path.basename(vars_dir)
 
         try:
             for item in os.listdir(settings.RESOURCES):
@@ -810,6 +841,346 @@ def import_archive(
             raise typer.Exit(1)
 
 
+def _generate_loopback_interfaces() -> list[dict]:
+    """Generate Loopback0 interfaces for eligible devices that don't have them."""
+    tasks = []
+
+    # Initialize NetBox API connection
+    netbox_api = pynetbox.api(settings.URL, token=settings.TOKEN)
+    if settings.IGNORE_SSL_ERRORS:
+        netbox_api.http_session.verify = False
+
+    logger.info("Analyzing devices for Loopback0 interface creation...")
+
+    # Get all devices
+    all_devices = netbox_api.dcim.devices.all()
+
+    for device in all_devices:
+        # Determine if this device should have a Loopback0 interface
+        should_have_loopback = False
+
+        # Check if device has role and if it's in the list of roles that need Loopback0
+        if device.role:
+            device_role_slug = ""
+            if hasattr(device.role, "slug"):
+                device_role_slug = device.role.slug.lower()
+            elif hasattr(device.role, "name"):
+                device_role_slug = device.role.name.lower()
+
+            # Node roles always get Loopback0 interfaces
+            if device_role_slug in NETBOX_NODE_ROLES:
+                should_have_loopback = True
+            # Switch roles only get Loopback0 if they have sonic_parameters.hwsku custom field
+            elif device_role_slug in NETBOX_SWITCH_ROLES:
+                # Check for sonic_parameters custom field with hwsku
+                if hasattr(device, "custom_fields") and device.custom_fields:
+                    sonic_params = device.custom_fields.get("sonic_parameters")
+                    if (
+                        sonic_params
+                        and isinstance(sonic_params, dict)
+                        and sonic_params.get("hwsku")
+                    ):
+                        should_have_loopback = True
+                        logger.debug(
+                            f"Switch {device.name} has sonic_parameters.hwsku: {sonic_params.get('hwsku')}"
+                        )
+                    else:
+                        logger.debug(
+                            f"Switch {device.name} does not have sonic_parameters.hwsku, skipping Loopback0"
+                        )
+
+        # Check if device type name contains "switch" (case insensitive)
+        if device.device_type and hasattr(device.device_type, "model"):
+            device_type_name = device.device_type.model.lower()
+            if "switch" in device_type_name:
+                # Also check for sonic_parameters.hwsku for device types containing "switch"
+                if hasattr(device, "custom_fields") and device.custom_fields:
+                    sonic_params = device.custom_fields.get("sonic_parameters")
+                    if (
+                        sonic_params
+                        and isinstance(sonic_params, dict)
+                        and sonic_params.get("hwsku")
+                    ):
+                        should_have_loopback = True
+                        logger.debug(
+                            f"Switch device {device.name} (type: {device_type_name}) has sonic_parameters.hwsku: {sonic_params.get('hwsku')}"
+                        )
+                    else:
+                        logger.debug(
+                            f"Switch device {device.name} (type: {device_type_name}) does not have sonic_parameters.hwsku, skipping Loopback0"
+                        )
+
+        if not should_have_loopback:
+            continue
+
+        # Create Loopback0 interface task (always create regardless of existence)
+        tasks.append(
+            {
+                "device_interface": {
+                    "device": device.name,
+                    "name": "Loopback0",
+                    "type": "virtual",
+                    "enabled": True,
+                    "tags": ["managed-by-osism"],
+                }
+            }
+        )
+        logger.info(f"Will create Loopback0 interface for device: {device.name}")
+
+    logger.info(f"Generated {len(tasks)} Loopback0 interface creation tasks")
+    return tasks
+
+
+def _get_cluster_segment_config_context(
+    netbox_api: pynetbox.api, cluster_id: int, cluster_name: str = ""
+) -> dict:
+    """
+    Retrieve the specific segment config context for a cluster via separate API call.
+
+    Each cluster has a config context assigned with the same name as the segment.
+    This function retrieves the content of that specific config context.
+
+    Args:
+        netbox_api: The NetBox API connection
+        cluster_id: The cluster ID to retrieve context for
+        cluster_name: Optional cluster name for logging
+
+    Returns:
+        dict: The configuration context data from the segment-specific config context
+    """
+    try:
+        logger.debug(
+            f"Retrieving segment config context for cluster {cluster_name} (ID: {cluster_id}) via separate API call"
+        )
+
+        # Get all config contexts that apply to this cluster
+        config_contexts = netbox_api.extras.config_contexts.filter(clusters=cluster_id)
+
+        # Look for the config context with the same name as the cluster (segment name)
+        segment_context = None
+        for ctx in config_contexts:
+            if ctx.name == cluster_name:
+                logger.debug(
+                    f"Found segment config context: '{ctx.name}' for cluster {cluster_name}"
+                )
+                segment_context = ctx
+                break
+
+        if segment_context and segment_context.data:
+            logger.info(
+                f"Retrieved segment config context '{segment_context.name}' for cluster {cluster_name}"
+            )
+
+            # Log the specific loopback configuration found
+            if "_loopback_network_ipv4" in segment_context.data:
+                logger.debug(
+                    f"Found loopback config in {segment_context.name}: IPv4={segment_context.data.get('_loopback_network_ipv4')}, IPv6={segment_context.data.get('_loopback_network_ipv6')}"
+                )
+
+            return segment_context.data
+        elif segment_context and not segment_context.data:
+            logger.warning(
+                f"Config context '{segment_context.name}' found for cluster {cluster_name} but contains no data"
+            )
+            return {}
+        else:
+            logger.warning(
+                f"No segment config context found for cluster {cluster_name} (expected config context with name '{cluster_name}')"
+            )
+            return {}
+
+    except Exception as e:
+        logger.error(
+            f"Error retrieving segment config context for cluster {cluster_name} (ID: {cluster_id}): {e}"
+        )
+        return {}
+
+
+def _generate_cluster_loopback_tasks() -> list[dict]:
+    """Generate loopback IP address assignments for devices with assigned clusters."""
+    tasks = []
+
+    # Initialize NetBox API connection
+    netbox_api = pynetbox.api(settings.URL, token=settings.TOKEN)
+    if settings.IGNORE_SSL_ERRORS:
+        netbox_api.http_session.verify = False
+
+    logger.info("Analyzing devices with clusters for loopback IP generation...")
+
+    # Get all devices with clusters assigned
+    devices_with_clusters = []
+    all_devices = netbox_api.dcim.devices.all()
+
+    for device in all_devices:
+        if device.cluster:
+            devices_with_clusters.append(device)
+            logger.debug(
+                f"Found device {device.name} with cluster {device.cluster.name}"
+            )
+
+    logger.info(f"Found {len(devices_with_clusters)} devices with assigned clusters")
+
+    # Group devices by cluster
+    clusters_dict = {}
+    for device in devices_with_clusters:
+        cluster_id = device.cluster.id
+        if cluster_id not in clusters_dict:
+            clusters_dict[cluster_id] = {"cluster": device.cluster, "devices": []}
+        clusters_dict[cluster_id]["devices"].append(device)
+
+    # Process each cluster
+    for cluster_id, cluster_data in clusters_dict.items():
+        cluster = cluster_data["cluster"]
+        devices = cluster_data["devices"]
+
+        logger.info(f"Processing cluster '{cluster.name}' with {len(devices)} devices")
+
+        # Get the segment-specific config context via separate API call
+        try:
+            config_context = _get_cluster_segment_config_context(
+                netbox_api, cluster_id, cluster.name
+            )
+            if not config_context:
+                logger.warning(
+                    f"Cluster '{cluster.name}' has no config context assigned, skipping loopback generation for {len(devices)} devices"
+                )
+                continue
+
+            # Extract loopback network configuration
+            loopback_ipv4_network = config_context.get("_loopback_network_ipv4")
+            loopback_ipv6_network = config_context.get("_loopback_network_ipv6")
+            loopback_offset_ipv4 = config_context.get("_loopback_offset_ipv4", 0)
+
+            if not loopback_ipv4_network:
+                logger.info(
+                    f"Cluster '{cluster.name}' has no _loopback_network_ipv4 in config context, skipping"
+                )
+                continue
+
+            logger.debug(
+                f"Cluster '{cluster.name}' config: IPv4={loopback_ipv4_network}, IPv6={loopback_ipv6_network}, offset={loopback_offset_ipv4}"
+            )
+
+            # Parse IPv4 network
+            try:
+                ipv4_network = ipaddress.IPv4Network(
+                    loopback_ipv4_network, strict=False
+                )
+            except ValueError as e:
+                logger.error(
+                    f"Invalid IPv4 network '{loopback_ipv4_network}' for cluster '{cluster.name}': {e}"
+                )
+                continue
+
+            # Parse IPv6 network if provided
+            ipv6_network = None
+            if loopback_ipv6_network:
+                try:
+                    ipv6_network = ipaddress.IPv6Network(
+                        loopback_ipv6_network, strict=False
+                    )
+                except ValueError as e:
+                    logger.error(
+                        f"Invalid IPv6 network '{loopback_ipv6_network}' for cluster '{cluster.name}': {e}"
+                    )
+
+            # Generate IP addresses for each device
+            for device in devices:
+                # Get device position (rack position)
+                position = getattr(device, "position", None)
+                if position is None:
+                    logger.warning(
+                        f"Device '{device.name}' has no rack position, skipping loopback generation"
+                    )
+                    continue
+
+                # Validate position is an integer
+                if not isinstance(position, int):
+                    try:
+                        position = int(position)
+                        logger.debug(
+                            f"Device '{device.name}' position converted from {type(getattr(device, 'position', None)).__name__} to int: {position}"
+                        )
+                    except (ValueError, TypeError) as e:
+                        logger.warning(
+                            f"Device '{device.name}' has invalid position '{getattr(device, 'position', None)}' (not convertible to int), skipping loopback generation: {e}"
+                        )
+                        continue
+
+                # Calculate IPv4 address: byte_4 = device_position * 2 - 1 + offset
+                byte_4 = position * 2 - 1 + loopback_offset_ipv4
+
+                try:
+                    # Convert network to list of octets and modify the last octet
+                    network_int = int(ipv4_network.network_address)
+                    device_ipv4_int = network_int + byte_4
+                    device_ipv4 = ipaddress.IPv4Address(device_ipv4_int)
+                    device_ipv4_with_mask = f"{device_ipv4}/32"
+
+                    # Generate IPv4 task
+                    tasks.append(
+                        {
+                            "ip_address": {
+                                "address": device_ipv4_with_mask,
+                                "assigned_object": {
+                                    "name": "Loopback0",
+                                    "device": device.name,
+                                },
+                            }
+                        }
+                    )
+                    logger.info(
+                        f"Generated IPv4 loopback: {device.name} -> {device_ipv4_with_mask}"
+                    )
+
+                    # Generate IPv6 address based on IPv4
+                    if ipv6_network:
+                        try:
+                            # Convert IPv4 to IPv6: fd93:363d:dab8:0:10:10:128:3/128
+                            ipv4_octets = str(device_ipv4).split(".")
+                            ipv6_suffix = f"{ipv4_octets[0]}:{ipv4_octets[1]}:{ipv4_octets[2]}:{ipv4_octets[3]}"
+
+                            # Create IPv6 address using the network prefix and IPv4-based suffix
+                            network_prefix = str(ipv6_network.network_address).rstrip(
+                                "::"
+                            )
+                            if network_prefix.endswith(":"):
+                                network_prefix = network_prefix.rstrip(":")
+                            device_ipv6 = f"{network_prefix}:0:{ipv6_suffix}/128"
+
+                            tasks.append(
+                                {
+                                    "ip_address": {
+                                        "address": device_ipv6,
+                                        "assigned_object": {
+                                            "name": "Loopback0",
+                                            "device": device.name,
+                                        },
+                                    }
+                                }
+                            )
+                            logger.info(
+                                f"Generated IPv6 loopback: {device.name} -> {device_ipv6}"
+                            )
+
+                        except Exception as e:
+                            logger.error(
+                                f"Error generating IPv6 address for device '{device.name}': {e}"
+                            )
+
+                except Exception as e:
+                    logger.error(
+                        f"Error generating IPv4 address for device '{device.name}': {e}"
+                    )
+
+        except Exception as e:
+            logger.error(f"Error processing cluster '{cluster.name}': {e}")
+            continue
+
+    logger.info(f"Generated {len(tasks)} cluster-based loopback IP assignment tasks")
+    return tasks
+
+
 def _generate_autoconf_tasks() -> list[dict]:
     """Generate automatic configuration tasks based on NetBox API data."""
     tasks = []
@@ -821,120 +1192,134 @@ def _generate_autoconf_tasks() -> list[dict]:
 
     logger.info("Analyzing NetBox data for automatic configuration...")
 
+    # Get all devices first and filter out switches
+    logger.info("Filtering out switch devices...")
+    all_devices = netbox_api.dcim.devices.all()
+    non_switch_devices = {}
+
+    for device in all_devices:
+        if device.role and hasattr(device.role, "slug"):
+            device_role_slug = device.role.slug.lower()
+        elif device.role and hasattr(device.role, "name"):
+            device_role_slug = device.role.name.lower()
+        else:
+            device_role_slug = ""
+
+        if device_role_slug not in NETBOX_SWITCH_ROLES:
+            non_switch_devices[device.id] = device
+
+    logger.info(
+        f"Found {len(non_switch_devices)} non-switch devices out of {len(all_devices)} total devices"
+    )
+
     # 1. MAC address assignment for interfaces
     logger.info("Checking interfaces for MAC address assignments...")
-    interfaces = netbox_api.dcim.interfaces.all()
 
-    for interface in interfaces:
-        # Skip virtual interfaces
-        if (
-            interface.type
-            and hasattr(interface.type, "value")
-            and "virtual" in interface.type.value.lower()
-        ):
-            continue
-        if (
-            interface.type
-            and hasattr(interface.type, "label")
-            and "virtual" in interface.type.label.lower()
-        ):
-            continue
+    for device_id, device in non_switch_devices.items():
+        # Get interfaces for this specific device
+        device_interfaces = netbox_api.dcim.interfaces.filter(device_id=device_id)
 
-        # Get MAC addresses for this interface
-        mac_addresses = netbox_api.ipam.mac_addresses.filter(interface_id=interface.id)
+        for interface in device_interfaces:
+            # Skip virtual interfaces
+            if (
+                interface.type
+                and hasattr(interface.type, "value")
+                and "virtual" in interface.type.value.lower()
+            ):
+                continue
+            if (
+                interface.type
+                and hasattr(interface.type, "label")
+                and "virtual" in interface.type.label.lower()
+            ):
+                continue
 
-        # If interface has exactly one MAC address and no primary MAC, assign it
-        if len(mac_addresses) == 1 and not interface.mac_address:
-            mac_addr = mac_addresses[0]
-            tasks.append(
-                {
-                    "device_interface": {
-                        "device": interface.device.name,
-                        "name": interface.name,
-                        "primary_mac_address": mac_addr.address,
+            # Check if interface has a MAC address that should be set as primary
+            mac_to_assign = None
+            if interface.mac_address:
+                # Use existing mac_address as primary_mac_address
+                mac_to_assign = interface.mac_address
+            elif interface.mac_addresses and not interface.mac_address:
+                # Use first MAC from mac_addresses list as primary
+                mac_to_assign = interface.mac_addresses[0]
+
+            if mac_to_assign:
+                tasks.append(
+                    {
+                        "device_interface": {
+                            "device": device.name,
+                            "name": interface.name,
+                            "primary_mac_address": mac_to_assign,
+                        }
                     }
-                }
-            )
-            logger.debug(
-                f"Found MAC assignment: {interface.device.name}:{interface.name} -> {mac_addr.address}"
-            )
+                )
+                logger.info(
+                    f"Found MAC assignment: {device.name}:{interface.name} -> {mac_to_assign}"
+                )
 
-    # 2. OOB IP assignment from eth0 interfaces
+    # 2. Consolidated device IP assignments (OOB, primary IPv4, primary IPv6)
+    logger.info("Checking for device IP assignments...")
+
+    # Dictionary to collect all device assignments by device name
+    device_assignments = {}
+
+    # Collect OOB IP assignments from eth0 interfaces
     logger.info("Checking eth0 interfaces for OOB IP assignments...")
-    eth0_interfaces = netbox_api.dcim.interfaces.filter(name="eth0")
-
-    for interface in eth0_interfaces:
-        # Get IP addresses assigned to this interface
-        ip_addresses = netbox_api.ipam.ip_addresses.filter(
-            assigned_object_id=interface.id
+    for device_id, device in non_switch_devices.items():
+        # Get eth0 interface for this specific device
+        eth0_interfaces = netbox_api.dcim.interfaces.filter(
+            device_id=device_id, name="eth0"
         )
 
-        for ip_addr in ip_addresses:
-            device = netbox_api.dcim.devices.get(interface.device.id)
-            # If device doesn't have OOB IP set, assign this IP
-            if not device.oob_ip:
-                tasks.append(
-                    {"device": {"name": device.name, "oob_ip": ip_addr.address}}
-                )
-                logger.debug(
+        for interface in eth0_interfaces:
+            # Get IP addresses assigned to this interface
+            ip_addresses = netbox_api.ipam.ip_addresses.filter(
+                assigned_object_id=interface.id
+            )
+
+            for ip_addr in ip_addresses:
+                if device.name not in device_assignments:
+                    device_assignments[device.name] = {"name": device.name}
+
+                device_assignments[device.name]["oob_ip"] = ip_addr.address
+                logger.info(
                     f"Found OOB IP assignment: {device.name} -> {ip_addr.address}"
                 )
 
-    # 3. Primary IPv4 assignment from Loopback0 interfaces
-    logger.info("Checking Loopback0 interfaces for primary IPv4 assignments...")
-    loopback_interfaces = []
-    loopback_interfaces.extend(netbox_api.dcim.interfaces.filter(name="Loopback0"))
-
-    for interface in loopback_interfaces:
-        # Get IPv4 addresses assigned to this interface
-        ip_addresses = netbox_api.ipam.ip_addresses.filter(
-            assigned_object_id=interface.id
+    # Collect primary IPv4 and IPv6 assignments from Loopback0 interfaces
+    logger.info("Checking Loopback0 interfaces for primary IP assignments...")
+    for device_id, device in non_switch_devices.items():
+        # Get Loopback0 interface for this specific device
+        loopback_interfaces = netbox_api.dcim.interfaces.filter(
+            device_id=device_id, name="Loopback0"
         )
 
-        for ip_addr in ip_addresses:
-            # Check if this is an IPv4 address
-            if ":" not in ip_addr.address:  # Simple IPv4 check
-                device = netbox_api.dcim.devices.get(interface.device.id)
-                # If device doesn't have primary IPv4 set, assign this IP
-                if not device.primary_ip4:
-                    tasks.append(
-                        {
-                            "device": {
-                                "name": device.name,
-                                "primary_ip4": ip_addr.address,
-                            }
-                        }
-                    )
-                    logger.debug(
+        for interface in loopback_interfaces:
+            # Get IP addresses assigned to this interface
+            ip_addresses = netbox_api.ipam.ip_addresses.filter(
+                assigned_object_id=interface.id
+            )
+
+            for ip_addr in ip_addresses:
+                if device.name not in device_assignments:
+                    device_assignments[device.name] = {"name": device.name}
+
+                # Check if this is an IPv4 or IPv6 address
+                if ":" not in ip_addr.address:  # Simple IPv4 check
+                    device_assignments[device.name]["primary_ip4"] = ip_addr.address
+                    logger.info(
                         f"Found primary IPv4 assignment: {device.name} -> {ip_addr.address}"
                     )
-
-    # 4. Primary IPv6 assignment from Loopback0 interfaces
-    logger.info("Checking Loopback0 interfaces for primary IPv6 assignments...")
-    for interface in loopback_interfaces:
-        # Get IPv6 addresses assigned to this interface
-        ip_addresses = netbox_api.ipam.ip_addresses.filter(
-            assigned_object_id=interface.id
-        )
-
-        for ip_addr in ip_addresses:
-            # Check if this is an IPv6 address
-            if ":" in ip_addr.address:  # Simple IPv6 check
-                device = netbox_api.dcim.devices.get(interface.device.id)
-                # If device doesn't have primary IPv6 set, assign this IP
-                if not device.primary_ip6:
-                    tasks.append(
-                        {
-                            "device": {
-                                "name": device.name,
-                                "primary_ip6": ip_addr.address,
-                            }
-                        }
-                    )
-                    logger.debug(
+                else:  # IPv6 address
+                    device_assignments[device.name]["primary_ip6"] = ip_addr.address
+                    logger.info(
                         f"Found primary IPv6 assignment: {device.name} -> {ip_addr.address}"
                     )
 
+    # Create consolidated device tasks from collected assignments
+    for device_assignment in device_assignments.values():
+        tasks.append({"device": device_assignment})
+
     logger.info(f"Generated {len(tasks)} automatic configuration tasks")
     return tasks
 
@@ -944,6 +1329,12 @@ def _generate_autoconf_tasks() -> list[dict]:
 )
 def autoconf_command(
     output: Annotated[str, typer.Option(help="Output file path")] = "999-autoconf.yml",
+    loopback_output: Annotated[
+        str, typer.Option(help="Loopback interfaces output file path")
+    ] = "299-autoconf.yml",
+    cluster_loopback_output: Annotated[
+        str, typer.Option(help="Cluster-based loopback IPs output file path")
+    ] = "399-autoconf.yml",
     debug: Annotated[bool, typer.Option(help="Debug")] = False,
     dryrun: Annotated[
         bool, typer.Option(help="Dry run - show tasks but don't write file")
@@ -954,13 +1345,16 @@ def autoconf_command(
     This command analyzes the NetBox database and generates configuration tasks
     for common patterns:
 
-    1. Assign primary MAC addresses to interfaces that have exactly one MAC
-    2. Assign OOB IP addresses from eth0 interfaces to devices
-    3. Assign primary IPv4 addresses from Loopback0 interfaces to devices
-    4. Assign primary IPv6 addresses from Loopback0 interfaces to devices
+    1. Create Loopback0 interfaces for switches and devices with specific roles
+    2. Generate cluster-based loopback IP addresses for devices with assigned clusters
+    3. Assign primary MAC addresses to interfaces that have exactly one MAC
+    4. Assign OOB IP addresses from eth0 interfaces to devices
+    5. Assign primary IPv4 addresses from Loopback0 interfaces to devices
+    6. Assign primary IPv6 addresses from Loopback0 interfaces to devices
 
-    The tasks are written to a YAML file (default: 999-autoconf.yml) in the
-    standard netbox-manager resource format.
+    The loopback interface tasks are written to 299-autoconf.yml, cluster-based
+    loopback IP tasks are written to 399-autoconf.yml, and other tasks are written
+    to 999-autoconf.yml in the standard netbox-manager resource format.
     """
     # Initialize logger
     init_logger(debug)
@@ -969,35 +1363,126 @@ def autoconf_command(
     validate_netbox_connection()
 
     try:
-        # Generate tasks
-        tasks = _generate_autoconf_tasks()
+        # Generate loopback interface tasks
+        loopback_tasks = _generate_loopback_interfaces()
 
-        if not tasks:
-            logger.info("No automatic configuration tasks found")
-            return
+        # Generate cluster-based loopback IP tasks
+        cluster_loopback_tasks = _generate_cluster_loopback_tasks()
+
+        # Generate other autoconf tasks
+        other_tasks = _generate_autoconf_tasks()
 
         if dryrun:
-            logger.info("Dry run - would generate the following tasks:")
-            for task in tasks:
-                logger.info(f"  {yaml.dump(task, default_flow_style=False).strip()}")
+            if loopback_tasks:
+                logger.info(
+                    "Dry run - would generate the following loopback interface tasks:"
+                )
+                for task in loopback_tasks:
+                    logger.info(
+                        f"  {yaml.dump(task, default_flow_style=False).strip()}"
+                    )
+
+            if cluster_loopback_tasks:
+                logger.info(
+                    "Dry run - would generate the following cluster-based loopback IP tasks:"
+                )
+                for task in cluster_loopback_tasks:
+                    logger.info(
+                        f"  {yaml.dump(task, default_flow_style=False).strip()}"
+                    )
+
+            if other_tasks:
+                logger.info(
+                    "Dry run - would generate the following other autoconf tasks:"
+                )
+                for task in other_tasks:
+                    logger.info(
+                        f"  {yaml.dump(task, default_flow_style=False).strip()}"
+                    )
             return
 
-        # Ensure output directory exists
-        output_dir = (
-            os.path.dirname(output) if os.path.dirname(output) else settings.RESOURCES
-        )
-        if output_dir and not os.path.exists(output_dir):
-            os.makedirs(output_dir, exist_ok=True)
+        files_written = 0
 
-        # If output is just a filename, put it in the RESOURCES directory
-        if not os.path.dirname(output) and settings.RESOURCES:
-            output = os.path.join(settings.RESOURCES, output)
+        # Handle loopback interfaces file
+        if loopback_tasks:
+            # Ensure output directory exists for loopback file
+            loopback_output_dir = (
+                os.path.dirname(loopback_output)
+                if os.path.dirname(loopback_output)
+                else settings.RESOURCES
+            )
+            if loopback_output_dir and not os.path.exists(loopback_output_dir):
+                os.makedirs(loopback_output_dir, exist_ok=True)
 
-        # Write tasks to YAML file
-        with open(output, "w") as f:
-            yaml.dump(tasks, f, default_flow_style=False, sort_keys=False)
+            # If loopback_output is just a filename, put it in the RESOURCES directory
+            if not os.path.dirname(loopback_output) and settings.RESOURCES:
+                loopback_output = os.path.join(settings.RESOURCES, loopback_output)
 
-        logger.info(f"Generated {len(tasks)} tasks in {output}")
+            # Write loopback tasks to YAML file
+            with open(loopback_output, "w") as f:
+                yaml.dump(loopback_tasks, f, default_flow_style=False, sort_keys=False)
+
+            logger.info(
+                f"Generated {len(loopback_tasks)} loopback interface tasks in {loopback_output}"
+            )
+            files_written += 1
+
+        # Handle cluster-based loopback IP tasks file
+        if cluster_loopback_tasks:
+            # Ensure output directory exists for cluster loopback file
+            cluster_loopback_output_dir = (
+                os.path.dirname(cluster_loopback_output)
+                if os.path.dirname(cluster_loopback_output)
+                else settings.RESOURCES
+            )
+            if cluster_loopback_output_dir and not os.path.exists(
+                cluster_loopback_output_dir
+            ):
+                os.makedirs(cluster_loopback_output_dir, exist_ok=True)
+
+            # If cluster_loopback_output is just a filename, put it in the RESOURCES directory
+            if not os.path.dirname(cluster_loopback_output) and settings.RESOURCES:
+                cluster_loopback_output = os.path.join(
+                    settings.RESOURCES, cluster_loopback_output
+                )
+
+            # Write cluster loopback tasks to YAML file
+            with open(cluster_loopback_output, "w") as f:
+                yaml.dump(
+                    cluster_loopback_tasks, f, default_flow_style=False, sort_keys=False
+                )
+
+            logger.info(
+                f"Generated {len(cluster_loopback_tasks)} cluster-based loopback IP tasks in {cluster_loopback_output}"
+            )
+            files_written += 1
+
+        # Handle other autoconf tasks file
+        if other_tasks:
+            # Ensure output directory exists
+            output_dir = (
+                os.path.dirname(output)
+                if os.path.dirname(output)
+                else settings.RESOURCES
+            )
+            if output_dir and not os.path.exists(output_dir):
+                os.makedirs(output_dir, exist_ok=True)
+
+            # If output is just a filename, put it in the RESOURCES directory
+            if not os.path.dirname(output) and settings.RESOURCES:
+                output = os.path.join(settings.RESOURCES, output)
+
+            # Write other tasks to YAML file
+            with open(output, "w") as f:
+                yaml.dump(other_tasks, f, default_flow_style=False, sort_keys=False)
+
+            logger.info(
+                f"Generated {len(other_tasks)} other autoconf tasks in {output}"
+            )
+            files_written += 1
+
+        if files_written == 0:
+            logger.info("No automatic configuration tasks found")
 
     except pynetbox.RequestError as e:
         logger.error(f"NetBox API error: {e}")
